"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const http_1 = require("http");
const https_1 = require("https");
const url_1 = require("url");
const async_enumerable_1 = require("@reactive-js/async-enumerable");
const disposable_1 = require("@reactive-js/disposable");
const http_2 = require("@reactive-js/http");
const node_1 = require("@reactive-js/node");
const observable_1 = require("@reactive-js/observable");
const pipe_1 = require("@reactive-js/pipe");
const httpContentEncoding_1 = require("./httpContentEncoding");
const httpContent_1 = require("./httpContent");
const httpResponse_1 = require("./httpResponse");
const spyScanner = ([uploaded, total], ev) => ev.type === 1
    ? [ev.chunk.length, total + uploaded]
    : [-1, total + uploaded];
const createOnSubscribe = (request, send, encodeContent, decodeHttpContentResponse) => (subscriber) => {
    var _a;
    subscriber.dispatch({ type: 1, request });
    const req = send();
    const onError = (cause) => {
        subscriber.dispose({ cause });
    };
    req.on("error", onError);
    const onResponse = (msg) => {
        subscriber.add(() => msg.destroy());
        const response = pipe_1.pipe(http_2.parseHttpResponseFromHeaders(msg.statusCode || -1, msg.headers, node_1.createReadableAsyncEnumerable(() => msg)), decodeHttpContentResponse);
        const disposableResponse = disposable_1.createDisposableValue(response, _ => {
            msg.destroy();
        });
        subscriber.dispatch({
            type: 4,
            request,
            response: disposableResponse,
        });
    };
    req.on("response", onResponse);
    const reqBodyEnumerator = node_1.createWritableAsyncEnumerator(req, subscriber);
    const doOnNotify = ([count, total]) => {
        const ev = count < 0
            ? { type: 3, request }
            : { type: 2, request, total };
        subscriber.dispatch(ev);
    };
    const spyEnumerator = pipe_1.pipe(async_enumerable_1.identity(), async_enumerable_1.lift(pipe_1.compose(observable_1.scan(spyScanner, () => [0, 0]), observable_1.onNotify(doOnNotify)))).enumerateAsync(subscriber);
    const contentEnumerator = pipe_1.pipe(((_a = request.content) === null || _a === void 0 ? void 0 : _a.body) || node_1.emptyReadableAsyncEnumerable, async_enumerable_1.lift(observable_1.onNotify(ev => spyEnumerator.dispatch(ev))), encodeContent).enumerateAsync(subscriber);
    subscriber
        .add(reqBodyEnumerator)
        .add(spyEnumerator)
        .add(contentEnumerator)
        .add(_ => {
        req.abort();
        req.removeAllListeners();
        req.destroy();
    });
    const onContinue = () => {
        contentEnumerator.subscribe(reqBodyEnumerator);
        reqBodyEnumerator.subscribe(contentEnumerator);
    };
    if (request.expectContinue) {
        req.on("continue", onContinue);
    }
    else {
        onContinue();
    }
};
const identity = (x) => x;
const requestIsCompressible = (request) => {
    const { content } = request;
    return content !== undefined ? httpContent_1.contentIsCompressible(content) : false;
};
exports.creatHttpClient = (clientOptions = {}) => {
    const { agent, insecureHTTPParser, maxHeaderSize, shouldEncode: shouldEncodeOption } = clientOptions, zlibOptions = tslib_1.__rest(clientOptions, ["agent", "insecureHTTPParser", "maxHeaderSize", "shouldEncode"]);
    return (request, requestOptions = {}) => {
        const { acceptedEncodings = httpContentEncoding_1.supportedEncodings } = requestOptions;
        const contentEncoding = httpContentEncoding_1.getFirstSupportedEncoding(acceptedEncodings);
        const send = () => {
            const { content } = request;
            const newContent = content === undefined || contentEncoding === undefined
                ? content
                : Object.assign(Object.assign({}, content), { contentEncodings: [...content.contentEncodings, contentEncoding] });
            const requestWithAcceptEncodingsAndContentEncoding = Object.assign(Object.assign({}, request), { acceptedEncodings: httpContentEncoding_1.supportedEncodings, content: newContent });
            const { method, uri } = requestWithAcceptEncodingsAndContentEncoding;
            const headers = {};
            http_2.writeHttpRequestHeaders(requestWithAcceptEncodingsAndContentEncoding, (header, value) => (headers[header] = value));
            const nodeRequestUrl = uri instanceof url_1.URL ? uri : new url_1.URL(uri.toString());
            const nodeRequestOptions = {
                agent,
                headers,
                insecureHTTPParser,
                maxHeaderSize,
                method,
            };
            return uri.protocol === "https:"
                ? https_1.request(nodeRequestUrl, nodeRequestOptions)
                : uri.protocol === "http:"
                    ? http_1.request(nodeRequestUrl, nodeRequestOptions)
                    : (() => {
                        throw new Error();
                    })();
        };
        const shouldEncodeOptionResult = shouldEncodeOption !== undefined
            ? shouldEncodeOption(request)
            : undefined;
        const shouldEncode = shouldEncodeOptionResult !== undefined
            ? shouldEncodeOptionResult
            : requestIsCompressible(request);
        const contentEncoder = contentEncoding !== undefined && shouldEncode
            ? node_1.transform(httpContentEncoding_1.createEncodingCompressTransform(contentEncoding, zlibOptions))
            : identity;
        const decodeHttpContentResponseWithOption = httpResponse_1.decodeHttpContentResponse(zlibOptions);
        return observable_1.createObservable(createOnSubscribe(request, send, contentEncoder, decodeHttpContentResponseWithOption));
    };
};
const redirectCodes = [
    301,
    302,
    303,
    307,
    308,
];
exports.createDefaultHttpResponseHandler = (sendHttpRequest, maxRedirects = 10) => {
    const handleResponse = (status) => {
        var _a;
        if (status.type === 4) {
            const { request, response } = status;
            const { location, preferences, statusCode } = response.value;
            const acceptedEncodings = ((_a = preferences) === null || _a === void 0 ? void 0 : _a.acceptedEncodings) || [];
            const shouldRedirect = redirectCodes.includes(statusCode) &&
                location !== undefined &&
                maxRedirects > 0;
            const [newRequest, newAcceptedEncodings] = shouldRedirect
                ? [
                    http_2.createRedirectHttpRequest(response.value)(request),
                ]
                : statusCode === 417
                    ? [Object.assign(Object.assign({}, request), { expectContinue: false })]
                    : statusCode === 415 &&
                        acceptedEncodings.length > 0
                        ? [request, acceptedEncodings]
                        : [request];
            if (request !== newRequest || newAcceptedEncodings !== undefined) {
                response.dispose();
                return pipe_1.pipe(sendHttpRequest(newRequest, {
                    acceptedEncodings: newAcceptedEncodings,
                }), exports.createDefaultHttpResponseHandler(sendHttpRequest, maxRedirects - 1));
            }
        }
        return observable_1.ofValue(status);
    };
    return observable_1.concatMap(handleResponse);
};
//# sourceMappingURL=httpClient.js.map