"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const compressible_1 = tslib_1.__importDefault(require("compressible"));
const content_type_1 = tslib_1.__importDefault(require("content-type"));
const iconv_lite_1 = tslib_1.__importDefault(require("iconv-lite"));
const node_1 = require("@reactive-js/node");
const pipe_1 = require("@reactive-js/pipe");
const httpContentEncoding_1 = require("./httpContentEncoding");
exports.encodeHttpContent = (contentBody, encoding, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: pipe_1.pipe(body, node_1.transform(httpContentEncoding_1.createEncodingCompressTransform(encoding, options))), contentEncodings: [...contentEncodings, encoding], contentLength: -1 });
};
exports.decodeHttpContent = (contentBody, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentEncodings.length === 0 || contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: contentEncodings
                .map(encoding => httpContentEncoding_1.createEncodingDecompressTransform(encoding, options))
                .reduceRight((acc, decoder) => pipe_1.pipe(acc, node_1.transform(decoder)), body), contentEncodings: [] });
};
exports.createBufferHttpContent = (chunk, contentType) => ({
    body: node_1.createReadableAsyncEnumerableFromBuffer(chunk),
    contentEncodings: [],
    contentLength: chunk.length,
    contentType,
});
exports.createReadableHttpContent = (factory, contentType, contentLength = -1) => ({
    body: node_1.createReadableAsyncEnumerable(factory),
    contentEncodings: [],
    contentLength,
    contentType,
});
exports.createStringHttpContent = (content, contentType, charset = "utf-8") => {
    const buffer = iconv_lite_1.default.encode(content, charset);
    const contentTypeParsed = content_type_1.default.parse(contentType);
    const { type, parameters = {} } = contentTypeParsed;
    const newParameters = Object.assign(Object.assign({}, parameters), { charset });
    const newContentTypeParsed = {
        type,
        parameters: newParameters,
    };
    const newContentType = content_type_1.default.format(newContentTypeParsed);
    return exports.createBufferHttpContent(buffer, newContentType);
};
exports.contentIsCompressible = (content) => {
    var _a;
    const contentType = (_a = content) === null || _a === void 0 ? void 0 : _a.contentType;
    return (contentType !== undefined && compressible_1.default(contentType)) || false;
};
//# sourceMappingURL=httpContent.js.map