"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = require("@reactive-js/http");
const node_1 = require("@reactive-js/node");
const observable_1 = require("@reactive-js/observable");
const pipe_1 = require("@reactive-js/pipe");
const disposable_1 = require("@reactive-js/disposable");
const writeResponseMessage = (resp) => (response) => {
    resp.statusCode = response.statusCode;
    http_1.writeHttpResponseHeaders(response, (header, value) => resp.setHeader(header, value));
};
const writeResponseContentBody = (resp) => ({ content, }) => observable_1.createObservable(subscriber => {
    var _a;
    const contentReadableEnumerator = (((_a = content) === null || _a === void 0 ? void 0 : _a.body) || node_1.emptyReadableAsyncEnumerable).enumerateAsync(subscriber);
    const responseWritableEnumerator = node_1.createWritableAsyncEnumerator(resp, subscriber);
    subscriber.add(contentReadableEnumerator).add(responseWritableEnumerator);
    responseWritableEnumerator.add(subscriber);
    contentReadableEnumerator.subscribe(responseWritableEnumerator);
    responseWritableEnumerator.subscribe(contentReadableEnumerator);
});
const defaultOnError = (_) => observable_1.empty();
const destroy = (val) => {
    val.destroy();
};
exports.createHttpRequestListener = (handler, scheduler, options = {}) => {
    const { onError = defaultOnError } = options;
    const handleRequest = (disposableRequest, disposableResponse) => {
        const req = disposableRequest.value;
        const resp = disposableResponse.value;
        const { method, url: path = "/", headers, httpVersionMajor, httpVersionMinor, } = req;
        const body = node_1.createReadableAsyncEnumerable(() => req);
        const isTransportSecure = req.socket.encrypted || false;
        return pipe_1.pipe(http_1.parseHttpRequestFromHeaders({
            method: method,
            path,
            headers: headers,
            httpVersionMajor,
            httpVersionMinor,
            isTransportSecure,
            body,
        }), handler, observable_1.onNotify(writeResponseMessage(resp)), observable_1.await_(writeResponseContentBody(resp)), observable_1.catchError(onError));
    };
    return (req, resp) => pipe_1.pipe(observable_1.using(() => [
        disposable_1.createDisposableValue(req, destroy),
        disposable_1.createDisposableValue(resp, destroy),
    ], handleRequest), observable_1.subscribe(scheduler));
};
//# sourceMappingURL=httpRequestListener.js.map