"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const httpContent_1 = require("./httpContent");
const httpContentEncoding_1 = require("./httpContentEncoding");
const responseIsCompressible = (response) => {
    const { content } = response;
    return content !== undefined ? httpContent_1.contentIsCompressible(content) : false;
};
exports.encodeHttpResponse = (request, options = {}) => response => {
    const { shouldEncode: shouldEncodeOption } = options, zlibOptions = tslib_1.__rest(options, ["shouldEncode"]);
    const shouldEncodeOptionResult = shouldEncodeOption !== undefined
        ? shouldEncodeOption(request, response)
        : undefined;
    const shouldEncode = shouldEncodeOptionResult !== undefined
        ? shouldEncodeOptionResult
        : responseIsCompressible(response);
    const { preferences } = request;
    const acceptedEncodings = preferences !== undefined && shouldEncode
        ? preferences.acceptedEncodings
        : [];
    const { content, vary } = response;
    const encoding = httpContentEncoding_1.getFirstSupportedEncoding(acceptedEncodings || []);
    const encodeBody = encoding !== undefined && content !== undefined;
    return Object.assign(Object.assign({}, response), { content: encoding !== undefined && content !== undefined
            ? httpContent_1.encodeHttpContent(content, encoding, zlibOptions)
            : content, vary: encodeBody ? [...vary, "Accept-Encoding"] : vary });
};
exports.decodeHttpContentResponse = (options) => response => {
    const { content } = response;
    return Object.assign(Object.assign({}, response), { content: content !== undefined ? httpContent_1.decodeHttpContent(content, options) : undefined });
};
//# sourceMappingURL=httpResponse.js.map