import compressible from "compressible";
import contentTypeParser from "content-type";
import iconv from "iconv-lite";
import { createReadableAsyncEnumerableFromBuffer, transform, createReadableAsyncEnumerable, } from "@reactive-js/node";
import { pipe } from "@reactive-js/pipe";
import { createEncodingCompressTransform, createEncodingDecompressTransform, } from "./httpContentEncoding";
export const encodeHttpContent = (contentBody, encoding, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: pipe(body, transform(createEncodingCompressTransform(encoding, options))), contentEncodings: [...contentEncodings, encoding], contentLength: -1 });
};
export const decodeHttpContent = (contentBody, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentEncodings.length === 0 || contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: contentEncodings
                .map(encoding => createEncodingDecompressTransform(encoding, options))
                .reduceRight((acc, decoder) => pipe(acc, transform(decoder)), body), contentEncodings: [] });
};
export const createBufferHttpContent = (chunk, contentType) => ({
    body: createReadableAsyncEnumerableFromBuffer(chunk),
    contentEncodings: [],
    contentLength: chunk.length,
    contentType,
});
export const createReadableHttpContent = (factory, contentType, contentLength = -1) => ({
    body: createReadableAsyncEnumerable(factory),
    contentEncodings: [],
    contentLength,
    contentType,
});
export const createStringHttpContent = (content, contentType, charset = "utf-8") => {
    const buffer = iconv.encode(content, charset);
    const contentTypeParsed = contentTypeParser.parse(contentType);
    const { type, parameters = {} } = contentTypeParsed;
    const newParameters = Object.assign(Object.assign({}, parameters), { charset });
    const newContentTypeParsed = {
        type,
        parameters: newParameters,
    };
    const newContentType = contentTypeParser.format(newContentTypeParsed);
    return createBufferHttpContent(buffer, newContentType);
};
export const contentIsCompressible = (content) => {
    var _a;
    const contentType = (_a = content) === null || _a === void 0 ? void 0 : _a.contentType;
    return (contentType !== undefined && compressible(contentType)) || false;
};
//# sourceMappingURL=httpContent.js.map