import { writeHttpResponseHeaders, parseHttpRequestFromHeaders, } from "@reactive-js/http";
import { createWritableAsyncEnumerator, emptyReadableAsyncEnumerable, createReadableAsyncEnumerable, } from "@reactive-js/node";
import { await_, catchError, createObservable, onNotify, subscribe, using, empty, } from "@reactive-js/observable";
import { pipe } from "@reactive-js/pipe";
import { createDisposableValue, } from "@reactive-js/disposable";
const writeResponseMessage = (resp) => (response) => {
    resp.statusCode = response.statusCode;
    writeHttpResponseHeaders(response, (header, value) => resp.setHeader(header, value));
};
const writeResponseContentBody = (resp) => ({ content, }) => createObservable(subscriber => {
    var _a;
    const contentReadableEnumerator = (((_a = content) === null || _a === void 0 ? void 0 : _a.body) || emptyReadableAsyncEnumerable).enumerateAsync(subscriber);
    const responseWritableEnumerator = createWritableAsyncEnumerator(resp, subscriber);
    subscriber.add(contentReadableEnumerator).add(responseWritableEnumerator);
    responseWritableEnumerator.add(subscriber);
    contentReadableEnumerator.subscribe(responseWritableEnumerator);
    responseWritableEnumerator.subscribe(contentReadableEnumerator);
});
const defaultOnError = (_) => empty();
const destroy = (val) => {
    val.destroy();
};
export const createHttpRequestListener = (handler, scheduler, options = {}) => {
    const { onError = defaultOnError } = options;
    const handleRequest = (disposableRequest, disposableResponse) => {
        const req = disposableRequest.value;
        const resp = disposableResponse.value;
        const { method, url: path = "/", headers, httpVersionMajor, httpVersionMinor, } = req;
        const body = createReadableAsyncEnumerable(() => req);
        const isTransportSecure = req.socket.encrypted || false;
        return pipe(parseHttpRequestFromHeaders({
            method: method,
            path,
            headers: headers,
            httpVersionMajor,
            httpVersionMinor,
            isTransportSecure,
            body,
        }), handler, onNotify(writeResponseMessage(resp)), await_(writeResponseContentBody(resp)), catchError(onError));
    };
    return (req, resp) => pipe(using(() => [
        createDisposableValue(req, destroy),
        createDisposableValue(resp, destroy),
    ], handleRequest), subscribe(scheduler));
};
//# sourceMappingURL=httpRequestListener.js.map