import { __rest } from "tslib";
import { contentIsCompressible, encodeHttpContent, decodeHttpContent, } from "./httpContent";
import { getFirstSupportedEncoding } from "./httpContentEncoding";
const responseIsCompressible = (response) => {
    const { content } = response;
    return content !== undefined ? contentIsCompressible(content) : false;
};
export const encodeHttpResponse = (request, options = {}) => response => {
    const { shouldEncode: shouldEncodeOption } = options, zlibOptions = __rest(options, ["shouldEncode"]);
    const shouldEncodeOptionResult = shouldEncodeOption !== undefined
        ? shouldEncodeOption(request, response)
        : undefined;
    const shouldEncode = shouldEncodeOptionResult !== undefined
        ? shouldEncodeOptionResult
        : responseIsCompressible(response);
    const { preferences } = request;
    const acceptedEncodings = preferences !== undefined && shouldEncode
        ? preferences.acceptedEncodings
        : [];
    const { content, vary } = response;
    const encoding = getFirstSupportedEncoding(acceptedEncodings || []);
    const encodeBody = encoding !== undefined && content !== undefined;
    return Object.assign(Object.assign({}, response), { content: encoding !== undefined && content !== undefined
            ? encodeHttpContent(content, encoding, zlibOptions)
            : content, vary: encodeBody ? [...vary, "Accept-Encoding"] : vary });
};
export const decodeHttpContentResponse = (options) => response => {
    const { content } = response;
    return Object.assign(Object.assign({}, response), { content: content !== undefined ? decodeHttpContent(content, options) : undefined });
};
//# sourceMappingURL=httpResponse.js.map