/// <reference types="node" />
import { Agent } from "http";
import { ZlibOptions, BrotliOptions } from "zlib";
import { AsyncEnumerableLike } from "@reactive-js/async-enumerable";
import { DisposableValueLike } from "@reactive-js/disposable";
import { HttpContentEncoding, HttpContentRequestLike, HttpContentResponseLike } from "@reactive-js/http";
import { ReadableEvent, ReadableMode } from "@reactive-js/node";
import { ObservableLike, ObservableOperatorLike } from "@reactive-js/observable";
export declare const enum HttpClientRequestStatusType {
    Begin = 1,
    Uploaded = 2,
    UploadComplete = 3,
    ResponseReady = 4
}
export interface HttpClientRequestStatusBegin {
    readonly type: HttpClientRequestStatusType.Begin;
    readonly request: HttpContentRequestLike<AsyncEnumerableLike<ReadableMode, ReadableEvent>>;
}
export interface HttpClientRequestStatusUploading {
    readonly type: HttpClientRequestStatusType.Uploaded;
    readonly request: HttpContentRequestLike<AsyncEnumerableLike<ReadableMode, ReadableEvent>>;
    readonly total: number;
}
export interface HttpClientRequestStatusUploadComplete {
    readonly type: HttpClientRequestStatusType.UploadComplete;
    readonly request: HttpContentRequestLike<AsyncEnumerableLike<ReadableMode, ReadableEvent>>;
}
export interface HttpClientRequestStatusResponseReady {
    readonly type: HttpClientRequestStatusType.ResponseReady;
    readonly request: HttpContentRequestLike<AsyncEnumerableLike<ReadableMode, ReadableEvent>>;
    readonly response: DisposableValueLike<HttpContentResponseLike<AsyncEnumerableLike<ReadableMode, ReadableEvent>>>;
}
export declare type HttpClientRequestStatus = HttpClientRequestStatusBegin | HttpClientRequestStatusUploading | HttpClientRequestStatusUploadComplete | HttpClientRequestStatusResponseReady;
export interface HttpClientOptions extends BrotliOptions, ZlibOptions {
    readonly agent?: Agent | boolean;
    readonly insecureHTTPParser?: boolean;
    readonly maxHeaderSize?: number;
    readonly shouldEncode?: (req: HttpContentRequestLike<unknown>) => boolean | undefined;
}
export interface HttpClientRequestOptions {
    readonly acceptedEncodings?: readonly HttpContentEncoding[];
}
export interface HttpClient {
    (request: HttpContentRequestLike<AsyncEnumerableLike<ReadableMode, ReadableEvent>>, requestOptions?: HttpClientRequestOptions): ObservableLike<HttpClientRequestStatus>;
}
export declare const creatHttpClient: (clientOptions?: HttpClientOptions) => HttpClient;
export declare const createDefaultHttpResponseHandler: (sendHttpRequest: HttpClient, maxRedirects?: number) => ObservableOperatorLike<HttpClientRequestStatus, HttpClientRequestStatus>;
//# sourceMappingURL=httpClient.d.ts.map